@@ -1,4 +1,4 @@
-# Copyright (C) 2003-2016 John Goerzen & contributors
+# Copyright (C) 2003-2019 John Goerzen & contributors
 #
 #    This program is free software; you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
@@ -34,9 +34,12 @@ SYNC_MUTEXES = {}
 SYNC_MUTEXES_LOCK = Lock()
 
 try:
-    import fcntl
+    import portalocker
 except:
-    pass # Ok if this fails, we can do without.
+    try:
+        import fcntl
+    except:
+        pass # Ok if this fails, we can do without.
 
 # FIXME: spaghetti code alert!
 def getaccountlist(customconfig):
@@ -232,10 +235,13 @@ class SyncableAccount(Account):
 
         self._lockfd = open(self._lockfilepath, 'w')
         try:
-            fcntl.lockf(self._lockfd, fcntl.LOCK_EX|fcntl.LOCK_NB)
+            portalocker.lock(self._lockfd, portalocker.LOCK_EX)
         except NameError:
-            #fcntl not available (Windows), disable file locking... :(
-            pass
+            # portalocker not available for Windows.
+            try:
+                fcntl.lockf(self._lockfd, fcntl.LOCK_EX|fcntl.LOCK_NB)
+            except NameError:
+                pass # fnctl not available, disable file locking... :(
         except IOError:
             self._lockfd.close()
             six.reraise(OfflineImapError,
@@ -250,6 +256,10 @@ class SyncableAccount(Account):
 
         #If we own the lock file, delete it
         if self._lockfd and not self._lockfd.closed:
+            try:
+                portalocker.unlock(self._lockfd)
+            except NameError:
+                pass
             self._lockfd.close()
             try:
                 os.unlink(self._lockfilepath)
@@ -375,24 +385,24 @@ class SyncableAccount(Account):
                                   "[%s]"% (remotefolder.getname(), remoterepos))
                     continue # Ignore filtered folder.
 
-                # The remote folder names must not have the local sep char in
-                # their names since this would cause troubles while converting
-                # the name back (from local to remote).
-                sep = localrepos.getsep()
-                if (sep != os.path.sep and
-                    sep != remoterepos.getsep() and
-                    sep in remotefolder.getname()):
-                    self.ui.warn('', "Ignoring folder '%s' due to unsupported "
-                        "'%s' character serving as local separator."%
-                        (remotefolder.getname(), localrepos.getsep()))
-                    continue # Ignore unsupported folder name.
-
                 localfolder = self.get_local_folder(remotefolder)
                 if not localfolder.sync_this:
                     self.ui.debug('', "Not syncing filtered folder '%s'"
                                  "[%s]"% (localfolder.getname(), localfolder.repository))
                     continue # Ignore filtered folder.
 
+                # The remote folder names must not have the local sep char in
+                # their names since this would cause troubles while converting
+                # the name back (from local to remote).
+                sep = localrepos.getsep()
+                if (sep == os.path.sep or
+                    sep == remoterepos.getsep() or
+                    sep in remotefolder.getname()):
+                    self.ui.warn('', "Ignoring folder '%s' due to unsupported "
+                        "'%s' character serving as local separator."%
+                        (remotefolder.getname(), localrepos.getsep()))
+                    continue # Ignore unsupported folder name.
+
                 if not globals.options.singlethreading:
                     thread = InstanceLimitedThread(
                         limitNamespace="%s%s"% (
@@ -511,6 +521,11 @@ def syncfolder(account, remotefolder, quick):
         uids = remotefolder.getmessageuidlist()
         localfolder.dropmessagelistcache()
         if len(uids) > 0:
+            # Reload the remote message list from min_uid. This avoid issues for
+            # old messages, which has been added from local on any previous run
+            # (IOW, message is older than maxage _and_ has high enough UID).
+            remotefolder.dropmessagelistcache()
+            remotefolder.cachemessagelist(min_uid=min(uids))
             localfolder.cachemessagelist(min_uid=min(uids))
         else:
             # Remote folder UIDs list is empty for the given range. We still
@@ -518,8 +533,12 @@ def syncfolder(account, remotefolder, quick):
             # emails).
             localfolder.cachemessagelist(min_date=date)
             uids = localfolder.getmessageuidlist()
+            # Take care to only consider positive uids. Negative UIDs might be
+            # present due to new emails.
+            uids = [uid for uid in uids if uid > 0]
             if len(uids) > 0:
                 # Update the remote cache list for this new min(uids).
+                remotefolder.dropmessagelistcache()
                 remotefolder.cachemessagelist(min_uid=min(uids))
 
     def cachemessagelists_startdate(new, partial, date):
